/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "usfm.h"
#include "book.h"
#include "bible.h"
#include "usfmtools.h"
#include "dialogunicode.h"


Book::Book(const string& path, bool split_lines) {
  /*
    This version is for use in a script application,
    or where we do not wish to check for conformity to Unicode.
   */
  load_execute(false, path, split_lines);
}


Book::Book(Gtk::Window& parent, const string& path, bool split_lines) {
  /*
    This version is for use in the gui application.
    As we are in the gui we will check for conformity to Unicode UTF-8.
    If it does not conform, we'll present the user with a list of possible conversions.
    The conversion is done, the file written, and then loaded as usual.
   */
  // See whether the file contents is proper Unicode
  string file_contents;
  bool unicode_valid = true;
  try {
    file_contents = Glib::file_get_contents(path);
  }
  catch (Glib::FileError) {
  	unicode_valid = false;
  }
  Glib::ustring unicode_file_contents;
  unicode_file_contents = file_contents;
  if (!unicode_file_contents.validate())
    unicode_valid = false;
  if (!unicode_valid) {
    // Unicode is not valid: Let the user convert it to proper Unicode, and then proceed.
    int result = Gtk::RESPONSE_NONE;
    // Conversion will be repeated if the user closes the dialog too early.
    while (result != Gtk::RESPONSE_OK) {
      {	
        UnicodeDialog unicodedialog(parent, path);
        result = unicodedialog.run();
      }
    }
  }
  load_execute(true, path, split_lines);
}


Book::~Book() {
}


void Book::load_execute(bool gui, const string& path, bool split_lines) {
  /*
     Loads the new book.
     It looks at the book in file "path", and provides information on it.
  */
  mypath = path;
  // Load, clean and split the lines.
  // The way to do it depends on whether we are loading a text file or a compressed file.
  Glib::ustring s;
  if (Glib::str_has_suffix(mypath, ".gz")) {
    FILE *stream;
    s = "gunzip -c '";
    s.append (mypath);
    s.append ("'");
    stream = popen(s.c_str(), "r");
    char buf[10240];
    while (fgets (buf, sizeof(buf), stream)) {
      s = buf;
      split_clean_lines (s, cleaned_lines, split_lines);
    }
    pclose(stream);
  } else {
    // Plain file, just load it.
    ReadText rt(mypath, true);
    for (unsigned int i = 0; i < rt.lines.size(); i++) {
      s = rt.lines[i];
      split_clean_lines (s, cleaned_lines, split_lines);
    }
  }
  // Go through the book to find out the number of chapters in it.
  Usfm usfm(0);
  number_of_chapters = 0;
  for (unsigned int i = 0; i < cleaned_lines.size(); i++) {
    s = cleaned_lines[i];
    s = usfm_extract_marker(s);
    if (usfm.is_chapter(s)) number_of_chapters++;
  }
  // Correct for some one-chapter book do not have the c marker at all.
  if (number_of_chapters == 0) number_of_chapters++;
  // Location for saving the backups of this book.
  backup_directory = Glib::build_filename(Glib::path_get_dirname(mypath), "backups");
  if (!Glib::file_test(backup_directory, Glib::FILE_TEST_IS_DIR))
    create_directory(backup_directory);  
}


unsigned int Book::number_of_verses (unsigned int chapternumber) {
  /*
    This function returns the number of verses that is found
    in chapter "chapternumber".
  */
  unsigned int begin, end;
  get_chapter_boundaries (chapternumber, begin, end);
  unsigned int numberofverses = 0;
  Usfm usfm(0);
  for (unsigned int i = begin; i <= end; i++) {
    Glib::ustring line = cleaned_lines[i];
    Glib::ustring marker = usfm_extract_marker (line);
    if (usfm.is_verse(marker))
      numberofverses++;
  }
  return numberofverses;
}


void Book::split_clean_lines (const Glib::ustring& text, vector<Glib::ustring>& lines, bool splitlines) {
  /*
Variable "text" brings in one line of text. This line is supposed to start with
an usfm. However, other usfms can be found within the line. There are
usfms that have also a closer marker. These can be left in the line. But the other
usfms cause the line of text to be split in multiple lines, which
are stored in variabele "lines".
However, if variabele "splitlines" is false, the splitting is not done.
   */
  Glib::ustring line = trim(text);
  if (splitlines) {
    // Split the lines on the usfm's.
    unsigned int position = 0;
    vector<unsigned int> cutpositions;
    Glib::ustring marker;
    Usfm usfm(0);
    position = usfm_search_marker_from (text, position);
    while (position != string::npos) {
      if (position > 0) {
        // A marker has been found, which is not at the beginning of the line.
        marker = text.substr(position + 1, usfm_search_marker_from_length (text, position) - 1);
        // Ensure that an unknown marker does not cut the line.
        if (usfm.is_known(marker)) {
          // Ensure that a marker with an endmarker does not cut the line.
          if (!usfm.has_endmarker(marker)) {
            cutpositions.push_back(position);
          }
        }
      }
      position = usfm_search_marker_from (text, ++position);
    }
    // At this point there is a vector that may either be empty or have cutting points.
    // If it is empty, no cuttings need be made, but if it has points, the line 
    // needs to be divided in pieces.
    Glib::ustring s;
    unsigned int previous_point = 0;
    for (unsigned int i = 0; i < cutpositions.size(); i++) {
      s = text.substr (previous_point, cutpositions[i] - previous_point);
      lines.push_back(trim(s));
      previous_point = cutpositions[i];
    }
    s = text.substr (previous_point, text.length());
    lines.push_back(trim(s));
  } else {
    // Do not split them.
    lines.push_back(line);
  }
}


void Book::get_chapter (unsigned int chapternumber, vector<Glib::ustring>& lines) {
  /*
    This function presents chapter "chapternumber" in "lines".
  */
  lines.clear();
  unsigned int begin, end;
  get_chapter_boundaries (chapternumber, begin, end);
  for (unsigned int i = begin; i <= end; i++) {
    lines.push_back(cleaned_lines[i]);
  }
  previously_presented_chapter = chapternumber; // Used later to accept the chapter again.
}


void Book::set_chapter (vector<Glib::ustring>& lines) {
  /*
    This function accepts the chapter in "lines", 
    and incorporates it again in the whole chapter,
    taking it as being the chapter number that was before presented.
  */
  // Get the two boundaries in the vector of strings, where the
  // chapter begins and where it ends.
  unsigned int begin, end;
  get_chapter_boundaries (previously_presented_chapter, begin, end);
  // Using an temporal vector of lines, start filling it with
  // 1. the set of lines before the current chapter starts.
  // 2. the current edited chapter.
  // 3. the lines following the current chapter.
  vector<string> temporal_lines;
  for (unsigned int i = 0; i < cleaned_lines.size(); i++) {
    if ((i < begin) || (i > end)) {
      temporal_lines.push_back(cleaned_lines[i]);
    } else {
      i = end;
      for (unsigned int i2 = 0; i2 < lines.size(); i2++) {
	temporal_lines.push_back(lines[i2]);
      }
    }
  }
  // Copy the temporal lines back to the book.
  cleaned_lines.clear();
  for (unsigned int i = 0; i < temporal_lines.size(); i++) {
    cleaned_lines.push_back(temporal_lines[i]);
  }
}


bool Book::save() {
  // Saves the book, and returns true if all went well.
  bool all_went_well = false;
  // Only save when we have got a filename.
  if (!mypath.empty()) {
    // Make backup of current file, if a directory is given where to put them.
    if (!backup_directory.empty()) {
      // Get the date and time in the right format.
      Shell shell0 ("date +%G-%m-%d-%H:%M-");
      shell0.run();
      string date = trim(shell0.get_output());
      // Get the filename to copy, and the path where to put it.
      string filename = Glib::path_get_basename(mypath);
      filename = Glib::build_filename (backup_directory, date + filename);
      // Carry out the copy command.
      string command = "cp '" + mypath + "' '" + filename + "'";
      Shell shell1(command);
      shell1.run();
      // Compress the backup file. Force it to overwrite an existing file.
      // It is done asynchronously, so that the gui does not block during the time it takes.
      command = "gzip -f '" + filename + "'";
      Shell shell2 (command);
      shell2.set_asynchronous();
      shell2.run();
    }
    try {
      WriteText wt(mypath);
      for (unsigned int i = 0; i < cleaned_lines.size(); i++) {
	wt.out << cleaned_lines[i] << endl;
      }
      all_went_well = true;
    }
    catch (exception &ex) {
      cerr << ex.what() << endl;
    }
  }
  return all_went_well;
}


void Book::get_chapter_boundaries (unsigned int chapter_number, unsigned int& begin, unsigned int& end) {
  /*
    This function finds the boundaries of the chapter number requested.
   */
  // Initialize the begin and end.
  begin = 0;
  end = 0;
  // Now go through all the text and start looking for the beginning and the end of the requested chapter.
  unsigned int CurrentChapter = 0;
  bool look_for_first_marker = true;
  Usfm usfm(0);
  for (unsigned int i = 0; i < cleaned_lines.size(); i++) {
    Glib::ustring s = cleaned_lines[i];
    s = usfm_extract_marker(s);
    if (CurrentChapter == 0) {
      // Allow for one-chapter books that have no \c 1 marker.
      if (usfm.is_verse(s)) CurrentChapter++;
    }
    if (usfm.is_chapter(s)) CurrentChapter++;
    if (CurrentChapter == chapter_number) {
      end = i;
      if (look_for_first_marker) {
	begin = i;
	look_for_first_marker = false;
      } 
    }
    // Speed up trick: break out when we have what we want.
    if (CurrentChapter > chapter_number) break;
  }
  // The first chapter also will include any headings. Care for that here.
  if (chapter_number == 1) 
    begin = 0;
}


vector<string> Book::get_backups () {
  /*
    Gets all the backups that exist for this book.
  */
  // This holds the values.
  vector<string> backups;
  // Get all the names.
  ReadFiles rd (backup_directory, "", "");
  for (unsigned int i = 0; i < rd.files.size(); i++) {
    backups.push_back(rd.files[i]);
  }
  return backups;
}


string Book::get_backup_directory() {
  return backup_directory;
}


unsigned int Book::get_previously_presented_chapter() {
  return previously_presented_chapter;
}


unsigned int Book::get_id_index() {
  /*
    This function returns the index of the book in the IDs.
   */
  unsigned int end = 10;
  if (end > cleaned_lines.size())
    end = cleaned_lines.size();
  Glib::ustring marker;
  Glib::ustring line;
  Usfm usfm(0);
  for (unsigned int i = 0; i < end; i++) {
    line = cleaned_lines[i];
    marker = usfm_extract_marker(line);
    if (usfm.is_id(marker)) {
      try {
        string id = line.substr(0, 3);
        return paratext_id_to_index(id);
      }
      catch (exception &ex) {
	    cerr << ex.what() << ": Bad ID in " << line << endl;
	    return 0;
      }
    }
  }
  return 0;
}
