/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialoggotoreference.h"
#include "utilities.h"
#include "dialogscript.h"
#include "bible.h"


GotoReferenceDialog::GotoReferenceDialog(Gtk::Window& parent, const string& currentbook, const string& currentchapter, const string& currentverse) :
  Dialog ("Goto reference", parent, true, false)
{
  add_button(Gtk::Stock::JUMP_TO, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  set_default_response(Gtk::RESPONSE_OK);
  // Set up all the widgets
  vbox.pack_start(hbox);
  hbox.pack_start(label, 0, 0, 10);
  label.set_text("Go to");
  label.set_padding(0, 10);
  hbox.pack_start(entry, 0, 0, 10);
  get_vbox()->pack_start(vbox);
  // Focus the entry
  set_focus_child(entry);
  show_all_children();
  // Init variables
  newreference = false;
  oldbook = currentbook;
  oldchapter = currentchapter;
  oldverse = currentverse;
}


GotoReferenceDialog::~GotoReferenceDialog() {
}


void GotoReferenceDialog::on_response (int response_id) {
  /*
    This interprets the text the user has typed in the entry as a valid reference.
    If needed it will use information of the current reference to complete the info.
    For example, when the user types "1", it is interpreted as verse one of the current
    chapter of the current book. If he types "21 10" it is interpreted as the current
    book, chapter 21 verse 10. And so forth.
   */

  if (response_id == Gtk::RESPONSE_OK) {
    // Filter the response.
    string response = trim(entry.get_text());
    // Change a colon to a space.
    unsigned int position = response.find(":");
    while (position != string::npos) {
      response[position] = ' ';
      position = response.find(":");
    }
    // Change to upper case.
    response = upperCase(response);
    // Trim again.
    response = trim(response);
    // Divide the response in parts.
    istringstream r(response);
    string s;
    for (unsigned int i = 0; i < 4; i++) {
      s.clear();
      r >> s;
      if (!s.empty())
	input.push_back(s);
    }
    // Deal with cases when the user enters e.g. "1 Corinthians 10:1".
    if (input.size() >= 2) {
      // This is the case when the first one is a number ...
      if (number_in_string(input[0]) == input[0]) {
	// ... and the second one contains no numbers at all.
	if (number_in_string(input[1]).empty()) {
	  input[1] = input[0] + " " + input[1];
	  input.erase(input.begin());
	}
      }
    }
    // See whether the first one is a name or a number.
    // If it's a name, then this will be the bookname.
    // If it's a number, there will be no bookname, but we have chapter and/or verse only.
    bool book_was_given = false;
    if (input.size() > 0) {
      if (number_in_string(input[0]) == input[0]) {
	// It's a number, so no bookname given. Use the current book.
	newbook = oldbook;
      } else {
	// It's a bookname, find out which.
	book_was_given = true;
	newbook = find_valid_book(input[0]);
	if (newbook.empty()) {
	  // If something like "1 co" has been added, deal with that by making it "1co", and checking again.
	  string s = input[0];
	  if (s.length() > 2) {
	    if (s[1] == ' ') {
	      s.erase(1, 1);
	      newbook = find_valid_book(s);
	    }
	  }
	  if (newbook.empty()) {
	    // Bad book: bail out.
	    show_bad_reference();
	    return;
	  }
	}
	// Remove the book, because it's no longer needed.
	input.erase(input.begin());
      }
    } else {
      show_bad_reference();
      return;
    }
    // As from here on we can be sure of this one thing: We've got a proper bookname.
    // Also as the bookname has been removed from the input data, only chapter and/or verse remain to be dealt with.
    if (input.size() >= 2) {
      // Two variabeles left, so that will be chapter and verse.
      newchapter = number_in_string(input[0]);
      newverse = number_in_string(input[1]);
      if (newchapter.empty() || newverse.empty()) {
	show_bad_reference();
	return;
      }
      newreference = true;
      return;
    }
    // Here we have only one variabele left.
    // This is either chapter (as in "Genesis 1") or verse (as in "1").
    // Or we have no variabele left, then we take chapter one verse one.
    if (book_was_given) {
      // Here deal with the fact that a book was given, but nothing else.
      if (input.size() == 0) {
	newchapter = "1";
	newverse = "1";
	newreference = true;
	return;
      }
    }
    // One variabele left, so we've either book or verse.
    if (book_was_given) {
      // Book was given, so it is a chapter.
      newchapter = number_in_string(input[0]);
      newverse = "1";
      if (newchapter.empty()) {
	show_bad_reference();
	return;
      }
      newreference = true;
      return;
    } else {
      // It is a verse.
      newverse = number_in_string(input[0]);
      newchapter = oldchapter;
      if (newverse.empty()) {
	show_bad_reference();
	return;
      }
      newreference = true;
      return;
    }
  }
}


string GotoReferenceDialog::find_valid_book(const string& rawbook) {
  /*
    This sees if variabele "rawbook" can be interpreted as a book in any way.
  */
  // Recognise the abbreviations used by Paratext.
  unsigned int index = paratext_id_to_index (rawbook);
  if (index != string::npos) {
    return index_to_biblebook(paratext_id_to_index(rawbook));
  }
  // See if shortening the bookname helps.
  if (rawbook.length() >= 3) {
    string s = rawbook.substr(0, 3);
    index = paratext_id_to_index(s);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  // Try the abbreviations of BibleWorks.
  index = bibleworks_id_to_index (rawbook);
  if (index != string::npos) {
    return index_to_biblebook(index);
  }
  // See if shortening the bookname helps.
  if (rawbook.length() >= 3) {
    string s = rawbook.substr(0, 3);
    index = bibleworks_id_to_index(s);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  // Try the abbreviations defined by the OSIS project.
  index = osis_id_to_index (rawbook);
  if (index != string::npos) {
    return index_to_biblebook(index);
  }
  // See if shortening the bookname helps.
  // The shortest abbreviation is 2 characters long,
  // and the longest 6. So we've to try them all.
  // Length: 2
  if (rawbook.length() >= 2) {
    string s = rawbook.substr(0, 2);
    index = osis_id_to_index(s);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  // Length: 3
  if (rawbook.length() >= 3) {
    string s = rawbook.substr(0, 3);
    index = osis_id_to_index(s);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  // Length: 4
  if (rawbook.length() >= 4) {
    string s = rawbook.substr(0, 4);
    index = osis_id_to_index(s);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  // Length: 5
  if (rawbook.length() >= 5) {
    string s = rawbook.substr(0, 5);
    index = osis_id_to_index(s);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  // Length: 6
  if (rawbook.length() >= 6) {
    string s = rawbook.substr(0, 6);
    index = osis_id_to_index(s);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  // If the book has not yet been found, then it's getting tough.
  // Check on names entered like Genesis or 1 Corinthians, the full English name
  index = english_id_to_index (rawbook);
  if (index != string::npos) {
    return index_to_biblebook(index);
  }
  // Not found yet, check on names like "1Corinthians".
  if (rawbook.length() >= 1) {
    string s = rawbook.substr(0, 1);
    string s2 = rawbook;
    if (s == "1" || s == "2" || s == "3") {
      s2.insert(1, " ");
    }
    index = english_id_to_index (s2);
    if (index != string::npos) {
      return index_to_biblebook(index);
    }
  }
  return "";
}


void GotoReferenceDialog::show_bad_reference() {
  // We did not find the reference: inform the user.
  Gtk::MessageDialog dialog(*this, "No such reference: " + entry.get_text());
  dialog.run();
}
