/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialoglanguage.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "languageutils.h"
#include "language.h"
#include "languages.h"
#include "directories.h"


LanguageDialog::LanguageDialog(Gtk::Window& parent, const string& languagename) :
  Dialog ("Language properties", parent, true, false)
{
  // Save variabeles.
  currentlanguagename = languagename;
  // Standard buttons.
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  // Language name.
  labelname.set_text("The name of the language");
  labelname.set_padding(5, 5);
  get_vbox()->pack_start(labelname, false, false, 0);
  entryname.set_text(currentlanguagename);
  get_vbox()->pack_start(entryname, false, false, 0);
  labelmessage.set_padding(5, 5);
  get_vbox()->pack_start(labelmessage, false, false, 0);
  // We only allow the name to be changed when the language is not in use.
  bool editable = (languageutils_lang_used_by(currentlanguagename).size() == 0);
  entryname.set_sensitive(editable);
  // Get info about the language.
  Language language (Glib::build_filename(directories_get_languages(), currentlanguagename));
  // Punctuation.
  labellanguage.set_text("The punctuation of the language");
  labellanguage.set_padding(5, 5);
  get_vbox()->pack_start(labellanguage, false, false, 0);
  entrypunctuation.set_text(language.punctuation);
  get_vbox()->pack_start(entrypunctuation, false, false, 0);
  // Show.
  show_all_children();
  // Handle gui.
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &LanguageDialog::set_gui), 100);
}


LanguageDialog::~LanguageDialog() {
}


bool LanguageDialog::set_gui() {
  bool sensitive = true;
  newlanguagename = trim(entryname.get_text());
  // Reject "New Language" as a name.
  if (newlanguagename == NEW_LANGUAGE) {
    sensitive = false;
    labelmessage.set_text("(the name of the language should be changed)");
  }
  // Reject empty names.
  if (newlanguagename.empty()) {
    sensitive = false;
    labelmessage.set_text("(the language needs a name)");
  }
  // Reject names that already exist.
  if (currentlanguagename != newlanguagename) {
    Languages languages(directories_get_languages());
    for (unsigned int i = 0; i < languages.languages.size(); i++)
      if (newlanguagename == languages.languages[i]) {
	sensitive = false;
	labelmessage.set_text("(a language with this name already exists)");
      }
  }
  // Deal with the sensitivity of the dialog.
  set_response_sensitive(Gtk::RESPONSE_OK, sensitive);
  if (sensitive)
    labelmessage.set_text("");
  return true;
}


void LanguageDialog::on_response (int response_id) {
  if (response_id == Gtk::RESPONSE_OK) {
    // If the name changed, carry this change out.
    if (currentlanguagename != newlanguagename) {
      string oldname = (Glib::build_filename(directories_get_languages(), currentlanguagename));
      string newname = (Glib::build_filename(directories_get_languages(), newlanguagename));
      string command = "mv \"" + Glib::strescape(oldname) + "\" \"" + Glib::strescape(newname) + "\"";
      Shell shell(command);
      shell.run();
    }
    // Save the settings belonging to the language.
    Language language (Glib::build_filename(directories_get_languages(), newlanguagename));
    language.punctuation = entrypunctuation.get_text();
  } else {
    // Remove the "New Language", because it was created but not used..
    string delname = (Glib::build_filename(directories_get_languages(), NEW_LANGUAGE));
    string command = "rm -rf '" + delname + "'";
    Shell shell(command);
    shell.run();
  }
}


