/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogproject.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "languageutils.h"
#include "languages.h"
#include "stylesheets.h"
#include "stylesheetutils.h"
#include "dialogselectbooks.h"
#include "scripture.h"
#include "bible.h"
#include <myconfig.h>
#include "directories.h"


ProjectDialog::ProjectDialog(Gtk::Window& parent, const string& projectname) :
  Dialog ("Project properties", parent, true, false)
{
  // Save variabeles.
  currentprojectname = projectname;
  // Standard buttons.
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  // Project name.
  labelname.set_text("The name of the project");
  labelname.set_padding(5, 5);
  get_vbox()->pack_start(labelname, false, false, 0);
  entryname.set_text(currentprojectname);
  get_vbox()->pack_start(entryname, false, false, 0);
  labelmessage.set_padding(5, 5);
  get_vbox()->pack_start(labelmessage, false, false, 0);
  // Get info about the project.
  Project ps (Glib::build_filename(directories_get_projects(), currentprojectname));
  // Project language.
  labellanguage.set_text("The language of the project");
  labellanguage.set_padding(5, 5);
  get_vbox()->pack_start(labellanguage, false, false, 0);
  combo_language_fill(ps.language);
  combolanguage.get_entry()->set_editable(false);
  get_vbox()->pack_start(combolanguage, false, false, 0);
  // Project stylesheet.
  vector<string> stylesheets;
  labelstylesheet.set_text("The stylesheet of the project");
  labelstylesheet.set_padding(5, 5);
  get_vbox()->pack_start(labelstylesheet, false, false, 0);
  combo_stylesheet_fill(ps.stylesheet);
  combostylesheet.get_entry()->set_editable(false);
  get_vbox()->pack_start(combostylesheet, false, false, 0);
  // Add and delete books buttons.
  buttonadd.set_label("Add book(s)");
  buttondelete.set_label("Delete book(s)");
  hbox.pack_start(buttonadd);
  buttonadd.signal_clicked().connect(sigc::mem_fun(*this, &ProjectDialog::on_book_add));
  hbox.pack_start(buttondelete);
  buttondelete.signal_clicked().connect(sigc::mem_fun(*this, &ProjectDialog::on_book_delete));
  get_vbox()->pack_start(hbox, false, false, 10);
  // Show.
  show_all_children();
  // Handle gui.
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &ProjectDialog::set_gui), 100);
}


ProjectDialog::~ProjectDialog() {
}


bool ProjectDialog::set_gui() {
  bool sensitive = true;
  newprojectname = trim(entryname.get_text());
  // Reject "New Project" as a name.
  if (newprojectname == NEW_PROJECT) {
    sensitive = false;
    labelmessage.set_text("(the name of the project should be changed)");
  }
  // Reject empty names.
  if (newprojectname.empty()) {
    sensitive = false;
    labelmessage.set_text("(the project needs a name)");
  }
  // Reject names that already exist.
  if (currentprojectname != newprojectname) {
    Projects projects(directories_get_projects());
    for (unsigned int i = 0; i < projects.projects.size(); i++)
      if (newprojectname == projects.projects[i]) {
	sensitive = false;
	labelmessage.set_text("(a project with this name already exists)");
      }
  }
  // Deal with the sensitivity of the dialog.
  set_response_sensitive(Gtk::RESPONSE_OK, sensitive);
  if (sensitive)
    labelmessage.set_text("");
  return true;
}


void ProjectDialog::on_response (int response_id) {
  if (response_id == Gtk::RESPONSE_OK) {
    // Deal with possible new projectname.
    if (currentprojectname != newprojectname) {
      string oldname = (Glib::build_filename(directories_get_projects(), currentprojectname));
      string newname = (Glib::build_filename(directories_get_projects(), newprojectname));
      string command = "mv \"" + Glib::strescape(oldname) + "\" \"" + Glib::strescape(newname) + "\"";
      Shell shell(command);
      shell.run();
    }
    // Save all settings.
    Project project (Glib::build_filename(directories_get_projects(), newprojectname));
    project.stylesheet = combostylesheet.get_entry()->get_text();
    project.language = combolanguage.get_entry()->get_text();
  } else {
    // Remove the "New Project", because it was created but not used..
    string delname = (Glib::build_filename(directories_get_projects(), NEW_PROJECT));
    string command = "rm -rf '" + delname + "'";
    Shell shell(command);
    shell.run();
  }
}


void ProjectDialog::combo_language_fill(const string& displayvalue) {
  Languages languages(directories_get_languages());
  combolanguage.set_popdown_strings(languages.languages);
  for (unsigned int i = 0; i < languages.languages.size(); i++) 
    if (displayvalue == languages.languages[i])
      combolanguage.get_entry()->set_text(displayvalue);
}


void ProjectDialog::combo_stylesheet_fill(const string& displayvalue) {
  Stylesheets stylesheets(directories_get_stylesheets());
  combostylesheet.set_popdown_strings(stylesheets.stylesheets);
  for (unsigned int i = 0; i < stylesheets.stylesheets.size(); i++) 
    if (displayvalue == stylesheets.stylesheets[i])
      combostylesheet.get_entry()->set_text(displayvalue);
}


void ProjectDialog::on_book_add() {
/*
 * This gives the user the possibility to add books that are still missing in 
 * the project. Only those books that are not yet in the project are shown.
 * If the user adds book(s), outlines of the book are placed in the project.
 * The outlines contains markers for chapters, verses, and the basics.
 * 
 * The outlines were taken from a text from BibleWorks, which was exported.
 * This exported text was converted by Bibledit Windows Companion to Paratext
 * format. Only the USFMs were written. Then with the Unix "rename" command
 * the names were changed, so that e.g. MAT.sfc became MAT.English.
 */
 
  vector<string> selectables;
  set<string> selection;
  Scripture scripture (currentprojectname);
  set<string> currentbooks;
  for (unsigned int i = 0; i < scripture.books.size(); i++)
    currentbooks.insert(scripture.books[i]);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
  	string book = index_to_biblebook(i);
  	if (currentbooks.find(book) == currentbooks.end())
  	  selectables.push_back(book);
  }
  SelectBooksDialog dialog (*this, sbAdd, selectables, selection);
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {
    for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
      string book = index_to_biblebook(i);
	  if (!(selection.find(book) == selection.end())) {
        string filename = projectutils_scripture_filename(currentprojectname, book);
        create_directory(Glib::path_get_dirname(filename));
        string templatename = Glib::build_filename(DATAPATH, index_to_paratext_id(i) + ".English");
        string command = "cp '" + templatename + "' '" + filename + "'";
	  	Shell shell (command);
	  	shell.run();
	  }
    }
  }
}


void ProjectDialog::on_book_delete() {
  vector<string> selectables;
  set<string> selection;
  Scripture scripture (currentprojectname);
  set<string> currentbooks;
  for (unsigned int i = 0; i < scripture.books.size(); i++)
    currentbooks.insert(scripture.books[i]);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
  	string book = index_to_biblebook(i);
  	if (currentbooks.find(book) != currentbooks.end())
  	  selectables.push_back(book);
  }
  SelectBooksDialog dialog (*this, sbDelete, selectables, selection);
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {
    if (selection.empty()) return;
    Gtk::MessageDialog dlg1(*this, "Are you sure you want to delete the books?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
    dlg1.set_default_response(Gtk::RESPONSE_NO);
    int result = dlg1.run();
    if (result != Gtk::RESPONSE_YES) return;
    Gtk::MessageDialog dlg2 (*this, "Are you really sure to delete something worth perhaps months of work?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
    dlg2.set_default_response(Gtk::RESPONSE_NO);
    result = dlg2.run();
    if (result != Gtk::RESPONSE_YES) return;
    for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
      // Remove the book.
      string book = index_to_biblebook(i);
	  if (selection.find(book) != selection.end()) {
        string filename = projectutils_scripture_filename(currentprojectname, book);
	  	Shell shell ("rm -rf '" + Glib::path_get_dirname(filename) + "'");
	  	shell.run();
	  }
    }
  }
}

