/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogstylesheet.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "stylesheetutils.h"
#include "stylesheet.h"
#include "stylesheets.h"
#include "directories.h"


StylesheetDialog::StylesheetDialog(Gtk::Window& parent, const string& stylesheetname) :
  Dialog ("Stylesheet properties", parent, true, false)
{
  // Save variabeles.
  currentstylesheetname = stylesheetname;
  // Standard buttons.
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  // Stylesheet name.
  labelname.set_text("The name of the stylesheet");
  labelname.set_padding(5, 5);
  get_vbox()->pack_start(labelname, false, false, 0);
  entryname.set_text(currentstylesheetname);
  get_vbox()->pack_start(entryname, false, false, 0);
  labelmessage.set_padding(5, 5);
  get_vbox()->pack_start(labelmessage, false, false, 0);
  // We only allow the name to be changed when the stylesheet is not in use.
  bool editable = (stylesheetutils_lang_used_by(currentstylesheetname).size() == 0);
  entryname.set_sensitive(editable);
  // Get info about the stylesheet.
  Stylesheet stylesheet (Glib::build_filename(directories_get_stylesheets(), currentstylesheetname));
  // Punctuation.
  labelstylesheet.set_text("The punctuation of the stylesheet");
  labelstylesheet.set_padding(5, 5);
  get_vbox()->pack_start(labelstylesheet, false, false, 0);
  entrypunctuation.set_text(stylesheet.punctuation);
  get_vbox()->pack_start(entrypunctuation, false, false, 0);
  // Show.
  show_all_children();
  // Handle gui.
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &StylesheetDialog::set_gui), 100);
}


StylesheetDialog::~StylesheetDialog() {
}


bool StylesheetDialog::set_gui() {
  bool sensitive = true;
  newstylesheetname = trim(entryname.get_text());
  // Reject "New Stylesheet" as a name.
  if (newstylesheetname == NEW_STYLESHEET) {
    sensitive = false;
    labelmessage.set_text("(the name of the stylesheet should be changed)");
  }
  // Reject empty names.
  if (newstylesheetname.empty()) {
    sensitive = false;
    labelmessage.set_text("(the stylesheet needs a name)");
  }
  // Reject names that already exist.
  if (currentstylesheetname != newstylesheetname) {
    Stylesheets stylesheets(directories_get_stylesheets());
    for (unsigned int i = 0; i < stylesheets.stylesheets.size(); i++)
      if (newstylesheetname == stylesheets.stylesheets[i]) {
	sensitive = false;
	labelmessage.set_text("(a stylesheet with this name already exists)");
      }
  }
  // Deal with the sensitivity of the dialog.
  set_response_sensitive(Gtk::RESPONSE_OK, sensitive);
  if (sensitive)
    labelmessage.set_text("");
  return true;
}


void StylesheetDialog::on_response (int response_id) {
  if (response_id == Gtk::RESPONSE_OK) {
    // If the name changed, carry this change out.
    if (currentstylesheetname != newstylesheetname) {
      string oldname = (Glib::build_filename(directories_get_stylesheets(), currentstylesheetname));
      string newname = (Glib::build_filename(directories_get_stylesheets(), newstylesheetname));
      string command = "mv \"" + Glib::strescape(oldname) + "\" \"" + Glib::strescape(newname) + "\"";
      Shell shell(command);
      shell.run();
    }
    // Save the settings belonging to the stylesheet.
    Stylesheet stylesheet (Glib::build_filename(directories_get_stylesheets(), newstylesheetname));
    stylesheet.punctuation = entrypunctuation.get_text();
  } else {
    // Remove the "New Stylesheet", because it was created but not used..
    string delname = (Glib::build_filename(directories_get_stylesheets(), NEW_STYLESHEET));
    string command = "rm -rf '" + delname + "'";
    Shell shell(command);
    shell.run();
  }
}


