/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogunicode.h"
#include "utilities.h"
#include "dialogscript.h"
#include "bible.h"
#include <sigc++/class_slot.h>


#define TEMPFILE "bibledit.unicode"
#define ANYTEXT "Rescue whatever text is available"


static bool object_lives = false;


UnicodeDialog::UnicodeDialog(Gtk::Window& parent, const string& filetoconvert) :
  Dialog ("Convert to UTF-8 Unicode", parent, true, false)
{
  closebutton = add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  closebutton->set_sensitive(false);
  set_default_response(Gtk::RESPONSE_OK);
  
  // First informative label
  label1.set_text("The book you are now opening is not in the right Unicode format.\n" \
		  "It needs to be converted before use.\n" \
		  "\n" \
		  "Looking up possible ways of converting it.\n" \
		  "Please wait ...");
  label1.set_padding(5, 5);
  get_vbox()->pack_start(label1, false, false, 0);
  // Progress bar.
  get_vbox()->pack_start(progressbar, false, false);
  // Set a label making space for the textview later.
  label2.set_text("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
  label2.set_padding(5, 5);
  get_vbox()->pack_start(label2, false, false, 0);
  // Get the whole list of possible encodings.
  FILE *stream;
  string s;
  s = "iconv --list";
  stream = popen(s.c_str(), "r");
  char buf[1024];
  while (fgets (buf, sizeof(buf), stream)) {
    s = buf;
    s = trim(s);
    if (Glib::str_has_suffix(s, "//"))
      s = s.substr(0, s.length() - 2);
    possible_encodings.push_back(s);
  }
  pclose(stream);
  // Store filename.
  filename = filetoconvert;
  
  // Show dialog.
  show_all_children();
  // Filter out the working encodings.
  current_encoding = 0;
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &UnicodeDialog::filter_encodings), 20);
  Glib::Thread::create(SigC::slot_class(*this, &UnicodeDialog::thread_filter_encodings), false);
  // Set variabeles
  object_lives = true;
}


UnicodeDialog::~UnicodeDialog() {
  unlink(temporary_file(TEMPFILE).c_str());
  object_lives = false;
}


void UnicodeDialog::on_response (int response_id) {
  /*
    Deal with response OK: copy the converted file.
   */
  if (response_id == Gtk::RESPONSE_OK) {
    string command;
    command = "cp --force ";
    command.append(temporary_file(TEMPFILE));
    command.append(" '");
    command.append(filename);
    command.append("'");
    Shell shell(command);
    shell.run();
  }
}


bool UnicodeDialog::filter_encodings() {
  /*
    This function manages the display while the thread looks up the encodings that actually would be able to convert the input file.
    Then it prepares the next screen.
  */
  // Set the progress bar.
  progressbar.set_fraction(double(current_encoding) / possible_encodings.size());
  // If all encodings have been dealt with, clean up, and present the next screen.
  if (current_encoding >= possible_encodings.size()) {
    // Clear some unused memory.
    possible_encodings.clear();
    // New screen, with new information.
    get_vbox()->remove(progressbar);
    get_vbox()->remove(label2);
    label1.set_text("Here is a list of possible encodings\n" \
		    "If one is selected, it will show the result as well\n" \
		    "When you're happy with the result, press OK.\n" \
		    "The book will then be converted.");
    working_encodings.push_back(ANYTEXT);
    combo.set_popdown_strings(working_encodings);
    combo.get_entry()->set_editable(false);
    combo.get_entry()->signal_changed().connect( sigc::mem_fun(*this, &UnicodeDialog::on_combo_changed) );
    combo.get_entry()->set_text(working_encodings[0]);
    get_vbox()->pack_start(combo, false, false);
    label3.set_text("");
    label3.set_padding(5, 5);
    get_vbox()->pack_start(label3, false, false, 0);
    get_vbox()->pack_start(scrolledwindow);
    scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    scrolledwindow.add(textview);
    textview.set_editable(false);
    show_all_children();
    on_combo_changed();
    // Allow window to close now.
    closebutton->set_sensitive(true);
    return false;
  } else {
    return true;
  }
}


void UnicodeDialog::thread_filter_encodings() {
  /*
    This thread look up the encodings that actually would be able to convert the input file.
    Conversions that don't work are removed.
  */
  while (current_encoding < possible_encodings.size()) {
    // The command to find out about the current encoding, whether it would work.
    string command;
    command = "iconv -f '";
    command.append(possible_encodings[current_encoding]);
    command.append("' -t UTF-8 '");
    command.append(filename);
    command.append("' > /dev/null 2>&1");
    Shell shell(command);
    shell.run();
    // Does our object still live, or has it been closed?
    if (!object_lives)
      return;
    // If conversion succeeded, store this encoding as a possible one.
    if (shell.result == 0) {
      working_encodings.push_back(possible_encodings[current_encoding]);
    }
    // Next encoding.
    current_encoding++;
  }
}


void UnicodeDialog::on_combo_changed() {
  /*
    If the user selects an encoding, this will show the example as well.
   */
  // Convert the text to a temporary file using the selected encoding.
  string s = combo.get_entry()->get_text();
  if (s.empty()) return;
  // See whether we'll use iconv or strings to filter the input text.
  string command;
  if (s == ANYTEXT) {
    command = "strings --bytes=2";
  } else {
    command = "iconv -f ";
    command.append(s);
    command.append(" -t UTF-8");
  }
  command.append(" '");
  command.append(filename);
  command.append("' > ");
  command.append(temporary_file(TEMPFILE));
  system(command.c_str());
  // Display the result.
  string contents;
  try {
    contents = Glib::file_get_contents(temporary_file(TEMPFILE));
  }
  catch (Glib::FileError) {
  }
  textview.get_buffer()->set_text(contents);
}


