/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "undo.h"


Undo::Undo(int dumy) {
  clear();
}


Undo::~Undo() {
}


void Undo::clear() {
  // Clear the whole undo stack.
  undo_position.clear();
  undo_text.clear();
  undo_insert.clear();
  non_undoable_action = false;
  pointer = 0;			// This is normally equal to the size of the stack.
}


void Undo::on_insert(const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes) {
  stack_operation(pos.get_offset(), text, true);
  shadow = pos.get_buffer()->get_text();
}


void Undo::on_erase(const Gtk::TextBuffer::iterator& range_begin, const Gtk::TextBuffer::iterator& range_end) {
  string::size_type length = shadow.length() - range_begin.get_buffer()->get_text().length();
  Glib::ustring erased_text = shadow.substr(range_begin.get_offset(), length);
  stack_operation(range_begin.get_offset(), erased_text, false);
  shadow = range_begin.get_buffer()->get_text();
}


void Undo::undo(Glib::RefPtr<Gtk::TextBuffer> buffer) {
  non_undoable_action = true;
  if (can_undo()) {
    pointer--;
    if (undo_insert[pointer]) {
      // Last action inserted, so we now erase it.
      Gtk::TextIter range_begin = buffer->get_iter_at_offset(undo_position[pointer] - undo_text[pointer].length());
      Gtk::TextIter range_end = buffer->get_iter_at_offset(undo_position[pointer]);
      buffer->erase(range_begin, range_end);
      // Place cursor at the edit.
      buffer->place_cursor(buffer->get_iter_at_offset(undo_position[pointer] - undo_text[pointer].length()));
    } else {
      // Last action erased, so we now insert it.
      Gtk::TextIter position = buffer->get_iter_at_offset(undo_position[pointer]);
      buffer->insert(position, undo_text[pointer]);
      // Place cursor at the edit.
      buffer->place_cursor(buffer->get_iter_at_offset(undo_position[pointer] + undo_text[pointer].length()));
    }
  }
  non_undoable_action = false;
}


void Undo::redo(Glib::RefPtr<Gtk::TextBuffer> buffer) {
  non_undoable_action = true;
  if (can_redo()) {
    if (undo_insert[pointer]) {
      // Last action was an insert, so we insert the same data again.
      Gtk::TextIter position = buffer->get_iter_at_offset(undo_position[pointer] - undo_text[pointer].length());
      buffer->insert(position, undo_text[pointer]);
      // Place cursor at the edit.
      buffer->place_cursor(buffer->get_iter_at_offset(undo_position[pointer]));
    } else {
      // Last action was an erase, so we erase the same data again.
      Gtk::TextIter range_begin = buffer->get_iter_at_offset(undo_position[pointer]);
      Gtk::TextIter range_end = buffer->get_iter_at_offset(undo_position[pointer] + undo_text[pointer].length());
      buffer->erase(range_begin, range_end);
      // Place cursor at the edit.
      buffer->place_cursor(buffer->get_iter_at_offset(undo_position[pointer]));
    }
    pointer++;
  }
  non_undoable_action = false;
}


bool Undo::can_undo() {
  return (pointer > 0);
}


bool Undo::can_redo() {
  return (pointer < undo_insert.size());
}


void Undo::stack_operation(int position, const Glib::ustring& text, bool insert) {
  /*
    This stores the insert or erase operation in the buffer,
    and manages the buffer.
   */
  if (non_undoable_action) return;
  // If the pointer does not point to the end of the buffer, erase from there to the end.
  if (pointer != undo_insert.size()) {
    int n = undo_insert.size() - pointer;
    for (int i = 0; i < n; i++) {
      undo_position.pop_back();
      undo_text.pop_back();
      undo_insert.pop_back();
    }
  }
  // Store the new data in the stack.
  undo_position.push_back(position);
  undo_text.push_back(text);
  undo_insert.push_back(insert);
  // We could be merging the current action with the most recent one,
  // if possible. This would be undoing / redoing whole words at once,
  // instead character by character. But this has not been implemented
  // at present, because I see little use for that. Perhaps later.

  // Trim buffer it if it grows too large.
  if (undo_insert.size() > 1000) {
    undo_position.erase(undo_position.begin());
    undo_text.erase(undo_text.begin());
    undo_insert.erase(undo_insert.begin());
  }
  // Update the pointer.
  pointer = undo_insert.size();
}

