 /*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <libgen.h>
#include <gtkmm.h>
#include <myconfig.h>
#include "constants.h" 


string trim(const string& s) {
  if (s.length() == 0)
    return s;
  unsigned int beg = s.find_first_not_of(" \t");	// Strip spaces, tabs and new lines
  unsigned int end = s.find_last_not_of(" \t\n\r");
  if (beg == string::npos)	// No non-spaces
    return "";
  return string (s, beg, end - beg + 1);
}


Glib::ustring trim(const Glib::ustring& s) {
  if (s.length() == 0)
    return s;
  unsigned int beg = s.find_first_not_of(" \t");	// Strip spaces, tabs and new lines
  unsigned int end = s.find_last_not_of(" \t\n\r");
  if (beg == string::npos)	// No non-spaces
    return "";
  return Glib::ustring (s, beg, end - beg + 1);
}


string number_in_string (const string& str) {
  // Looks for and returns a positive number in a string.
  string output = str;
  #define MY_NUMBERS "0123456789"
  output.erase(0, output.find_first_of(MY_NUMBERS));
  unsigned int end_position = output.find_first_not_of(MY_NUMBERS);
  #undef MY_NUMBERS
  if (end_position != string::npos) {
    output.erase(end_position, output.length());
  }
  return output;
}


string upperCase(const string& s) {
// Make an uppercase copy of s
  string upper(s);
  for(size_t i = 0; i < s.length(); ++i)
    upper[i] = toupper(upper[i]);
  return upper;
}


string lowerCase(const string& s) {
// Make a lowercase copy of s
  string lower(s);
  for(size_t i = 0; i < s.length(); ++i)
    lower[i] = tolower(lower[i]);
  return lower;
}


string remove_spaces(const string& s) {
  string s2 = s;
  unsigned int spacepos = s2.find(" ");
  while (spacepos != string::npos) {
    s2.erase(spacepos,1);
    spacepos = s2.find(" ");
  }
  return s2;
}


void create_directory (const string& dir) {
  // Creates directory, with the parents, if need be.
  // Function mkdir could be used (see man 2 mkdir), but this does not allow for 
  // the creation of the parent directories. The core utility mkdir provides
  // this functionality, so is preferred, and used here.
  string s;
  s = "mkdir -p '" + dir + "'";
  Shell shell(s);
  shell.run();
}


unsigned int convert_to_int (const string& str) {
  unsigned int i = 0;
  istringstream r(str);
  r >> i;
  return i;
}


string convert_to_string (int i) {
  ostringstream r;
  r << i;
  return r.str();
}


string convert_to_string (unsigned int& i) {
  int i2 = i;
  return convert_to_string(i2);
}


string convert_to_string (bool b) {
  if (b) return "1";
  else return "0";
}


bool convert_to_bool (const string& s) {
  if (s == "") return false;
  if (s == "0") return false;
  if (s == "F") return false;
  else return true;
}


void write_lines (const string& file, vector<string>& lines) {
  WriteText wt (file);
  for (unsigned int i = 0; i < lines.size(); i++) {
    wt.out << lines[i] << endl;
  }
}


void write_lines (const string& file, vector<Glib::ustring>& lines) {
  WriteText wt (file);
  for (unsigned int i = 0; i < lines.size(); i++) {
    wt.out << lines[i] << endl;
  }
}


string temporary_file(const string& filename) {
  return ((Glib::build_filename(Glib::get_tmp_dir(), filename)));
}


bool program_is_running (const string& commandline) {
  /* 
     Returns true if the program given on "commandline"
     is running, else false.
  */
  // Run the process status program.
  FILE * fd;
  fd = popen ("ps -ax", "r");
  if (!fd) {
    return false;
  }
  // Read the results of it.
  int i = 0;
  char buf[1024];
  while (fgets (buf, sizeof(buf), fd) != NULL) {
    if (strstr (buf, commandline.c_str()) != NULL)
      i++;
  }
  int result;
  result = pclose(fd);
  // Return true if one or more of the commandlines were found.
  // That is, return true if the program runs.
  return (i > 0);
}


bool string_in_file (const string& filename, const string& line) {
  /* 
     Returns true if the string "line" is found in file "filename".
   */
  string command = "grep --quiet '" + line + "' '" + filename + "' > /dev/null 2>&1";
  int i = system(command.c_str()); // Will return exitstatus 0 if a match was found.
  return (i == 0);
}


string string_reverse (const string& s) {
  string returnvalue;
  for (int i = s.length() - 1; i >= 0; i--)
    returnvalue.append(s.substr(i, 1));
  return returnvalue;
}


Glib::ustring double_apostrophy (const Glib::ustring& line) {
  /*
    SQLite needs any apostrophy in the data to be prefixed by another one.
    This function does that.
   */
  Glib::ustring returnvalue;
  returnvalue = line;
  string::size_type offset;
  offset = returnvalue.find("'");
  while (offset != string::npos) {
    returnvalue.insert(offset, "'");
    offset++;
    offset++;
    offset = returnvalue.find("'", offset);
  }
  return returnvalue;
}


unsigned int file_get_modification_time(const string& filename) {
  string command = "stat --format=%Y '";
  command.append(filename);
  command.append("'");
  Shell shell(command);
  shell.run();
  return convert_to_int(shell.get_output());
}


ReadDirectories::ReadDirectories (const string& path, const string& prefix, const string& suffix) {
  // Reads the directories in directory "path" that end on "suffix".
  // It does not return regular files.
  try {
    Glib::Dir dir (path);
    vector<string> entries (dir.begin(), dir.end());
    for (unsigned int i = 0; i < entries.size(); i++) {
      if (Glib::str_has_suffix(entries[i], suffix))
        if (Glib::str_has_prefix(entries[i], prefix))
          if (Glib::file_test(Glib::build_filename(path, entries[i]), Glib::FILE_TEST_IS_DIR))
            directories.push_back (entries[i]);
    }
  }
  catch(...) {
  }
}


ReadDirectories::~ReadDirectories() {
}


ReadFiles::ReadFiles (const string& path, const string& prefix, const string& suffix) {
  // Reads the regular files in directory "path" that end on "suffix".
  // It does not return directories.
  try {
    Glib::Dir dir (path);
    vector<string> entries (dir.begin(), dir.end());
    for (unsigned int i = 0; i < entries.size(); i++) {
      if (Glib::str_has_suffix(entries[i], suffix))
        if (Glib::str_has_prefix(entries[i], prefix))
          if (!Glib::file_test(Glib::build_filename(path, entries[i]), Glib::FILE_TEST_IS_DIR))
            files.push_back (entries[i]);
    }
  }
  catch(...) {
  }
}


ReadFiles::~ReadFiles() {
}


ReadText::ReadText (const string& file, bool silent) {
  // Reads the text and stores it line by line, trimmed, into "Lines".
  // If "silent" is true, then no exception will be thrown in case of an error.
  ifstream in(file.c_str());
  if(!in) {
    if (!silent) {
      cerr << "Error opening file " << file << endl;
      throw;
    }
    return;
  }
  string s;
  while (getline(in, s)) 
    lines.push_back(trim(s));
}


ReadText::~ReadText () {
}


WriteText::WriteText (const string& file, bool append) {
  /*
    This opens a textfile for writing.
    If 'append' is set, it appends the output to an existing file.
   */
  if (append) {
    out.open (file.c_str(), ios_base::out | ios_base::app);
  } else {
    out.open (file.c_str());
  }
  if(!out) {
    throw runtime_error("Error creating file " + file);
  }
}


WriteText:: ~WriteText() {
}


Parse::Parse (const Glib::ustring& line) {
  /*
    Parses a line of text in its separate words.
  */
  Glib::ustring processed_line;
  processed_line = trim(line);
  processed_line.append(" ");
  unsigned int spaceposition;
  spaceposition = processed_line.find(" ");
  while (spaceposition != string::npos) {
    Glib::ustring word = processed_line.substr(0, spaceposition);
    string::size_type location = word.find_last_of(".,;:");
    if (location != string::npos)
      word = word.substr(0, location);
    words.push_back(word);
    processed_line.erase(0, spaceposition+1);
    spaceposition = processed_line.find(" ");
  }
}


Parse::~Parse() {
}


Shell::Shell (const string& command) {
  /*
    Instead of using the expensive system() command,
    the program should use this Shell object.
    The system() function is expensive, because
    bibledit.bin is so large, and so the fork()
    command takes a lot of time.
  */
  mycommand = command;
  asynchronous = false;
  result = 0;
}


Shell::~Shell() {
}


void Shell::set_asynchronous() {
  asynchronous = true;
}


void Shell::run() {
  // Assemble and write the script that we wish to run.
  vector<string> script;
  script.push_back("#!/bin/sh");
  string outputfile;
  outputfile = Glib::get_home_dir() + "/.bibledit/temporary.script.output";
  mycommand.append (" > ");
  mycommand.append (outputfile);
  if (asynchronous)
    mycommand.append(" &");
  script.push_back(mycommand);
  script.push_back("echo $? > ~/.bibledit/temporary.script.result");
  string scriptfilename = Glib::get_home_dir() + "/.bibledit/temporary.script";
  write_lines(scriptfilename, script);
  // Send a character down the pipe to signal the shell process that it can run the script.
  int fd = open("/tmp/bibledit.pipe.sync", O_WRONLY, 0);
  if (fd < 0) {
    cerr << "Could not connect to pipe" << endl;
    return;
  }
  write (fd, "\n", 1);
  close (fd);
  // Wait for the temporary script to disappear, which is a sign it has finished.
  while (Glib::file_test(scriptfilename, Glib::FILE_TEST_EXISTS)) {
    Glib::usleep(1000);
  }
  // Get the result of the command.
  try {
    string filename = Glib::get_home_dir() + "/.bibledit/temporary.script.result";
    result = convert_to_int(Glib::file_get_contents(filename));
    unlink (filename.c_str());			    
  }
  catch (Glib::FileError) {
    result = -1;
  }
  // Get the output of the command.
  ReadText rt(outputfile, true);
  for (unsigned int i = 0; i < rt.lines.size(); i++)
    output_lines.push_back(rt.lines[i]);
  unlink(outputfile.c_str());
}


string Shell::get_output() {
  try {
    return output_lines[0];
  }
  catch (exception) {
    return "";
  }
}


