/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "worddatabase.h"
#include <myconfig.h>
#include "sqlite3_plus.h"
#include "usfmtools.h"
#include "bible.h"
#include "log.h"
#include "book.h"
#include "usfm.h"
#include "scripture.h"
#include <set>


#define TABLE_LINE_AS_IS "line_as_is"
#define TABLE_LINE_CASEFOLDED "line_casefolded"
#define TABLE_WORD_AS_IS "word_as_is"
#define TABLE_WORD_CASEFOLDED "word_casefolded"
#define TABLE_FILE_SIGNATURE "file_signature"


Worddatabase::Worddatabase (const string& project) {
  // Initialize variables.
  myproject = project;
}


Worddatabase::~Worddatabase () {
}


void Worddatabase::initialize() {
  try {
    // Connect to the database and create the table(s) if needed.
    sqlite3::connection con(dbfilename().c_str());
    int count;
    // Check for the table that stores the lines as they are.
    count = con.executeint32("select count(*) from sqlite_master where name='%s';", TABLE_LINE_AS_IS);
    if(count==0)
      con.executenonquery("create table %s (book integer, chapter integer, verse integer, line text);", TABLE_LINE_AS_IS);
    // Check for the table that stores the lines in casefolded form.
    count = con.executeint32("select count(*) from sqlite_master where name='%s';", TABLE_LINE_CASEFOLDED);
    if(count==0)
      con.executenonquery("create table %s (book integer, chapter integer, verse integer, line text);", TABLE_LINE_CASEFOLDED);
    // Check for the table that stores the words as they are.
    count = con.executeint32("select count(*) from sqlite_master where name='%s';", TABLE_WORD_AS_IS);
    if(count==0)
      con.executenonquery("create table %s (book integer, chapter integer, verse integer, line text);", TABLE_WORD_AS_IS);
    // Check for the table that stores the words in casefolded form.
    count = con.executeint32("select count(*) from sqlite_master where name='%s';", TABLE_WORD_CASEFOLDED);
    if(count==0)
      con.executenonquery("create table %s (book integer, chapter integer, verse integer, line text);", TABLE_WORD_CASEFOLDED);
    // Check for the table that stores the file signature
    count = con.executeint32("select count(*) from sqlite_master where name='%s';", TABLE_FILE_SIGNATURE);
    if(count==0)
      con.executenonquery("create table %s (filename text, signature text);", TABLE_FILE_SIGNATURE);
    con.close();
    // Get books in project.
    Scripture scripture(myproject);
    // Delete any books not in the project.
    set<string> currentbooks;
    for (unsigned int i = 0; i < scripture.books.size(); i++) {
      currentbooks.insert(scripture.books[i]);
    }
    for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
      if (currentbooks.find(index_to_biblebook(i)) == currentbooks.end()) {
        delete_book(i);  
      }
    }
    // Check each book.
    for (unsigned int i = 0; i < scripture.paths.size(); i++)
      check_book(scripture.paths[i]);
  }
  catch(exception &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
  catch(Glib::Error &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
}


void Worddatabase::update_chapter(unsigned int book, unsigned int chapter, const vector<Glib::ustring>& lines) {
  log ("Updating " + index_to_biblebook(book) + " " + convert_to_string(chapter));
  try {
    // Connect and start transaction
    sqlite3::connection con(dbfilename().c_str());
    con.executenonquery("begin;");
    // Delete previous data (if any);
    con.executenonquery("delete from '%s' where book = %d and chapter = %d;", TABLE_LINE_AS_IS, book, chapter);
    con.executenonquery("delete from '%s' where book = %d and chapter = %d;", TABLE_LINE_CASEFOLDED, book, chapter);
    con.executenonquery("delete from '%s' where book = %d and chapter = %d;", TABLE_WORD_AS_IS, book, chapter);
    con.executenonquery("delete from '%s' where book = %d and chapter = %d;", TABLE_WORD_CASEFOLDED, book, chapter);
    // Process each line.
    string currentverse = "1";
    Usfm usfm(0);
    for (unsigned int i = 0; i < lines.size(); i++) {
      Glib::ustring line (lines[i]);
      // Normalize the string for better result when comparing text.
      line = line.normalize();
      // Extract the verse number.
      Glib::ustring string_line;
      string_line = line.raw();
      Glib::ustring marker;
      marker = usfm_extract_marker(string_line);
      if (usfm.is_verse (marker)) 
      currentverse = number_in_string(string_line);
      // Any apostrophe (') should be prefixed by another one.
      line = double_apostrophy(line);
      // Store the data in the database.
      con.executenonquery("insert into '%s' values(%d, %d, %s, '%s');", 
			  TABLE_LINE_AS_IS, book, chapter, currentverse.c_str(), line.c_str());
      // Parse the line into separate words (keep them for later).
      Parse parse(line);
      // Casefold the line and store this also.
      line = line.casefold();
      con.executenonquery("insert into '%s' values(%d, %d, %s, '%s');", 
			  TABLE_LINE_CASEFOLDED, book, chapter, currentverse.c_str(), line.c_str());
      // Store the original parsed words, and the casefolded versions.
      for (unsigned int i = 0; i < parse.words.size(); i++) {
      con.executenonquery("insert into '%s' values(%d, %d, %s, '%s');", 
			    TABLE_WORD_AS_IS, book, chapter, currentverse.c_str(), parse.words[i].c_str());
      con.executenonquery("insert into '%s' values(%d, %d, %s, '%s');", 
			    TABLE_WORD_CASEFOLDED, book, chapter, currentverse.c_str(), parse.words[i].casefold().c_str());
      }
    }
    con.executenonquery("commit;");
    con.close();
  }
  catch(exception &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
  catch(Glib::Error &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
}


void Worddatabase::log (const string& message) {
  log_message("Words database: " + message);
}


void Worddatabase::check_book(const string& filename) {
  try {
    // Connect to the database
    sqlite3::connection con(dbfilename().c_str());
    // Get the actual signature for the file.
    string actual_signature;
    FILE *stream;
    string s = "ls -l '";
    s.append (filename);
    s.append ("'");
    stream = popen(s.c_str(), "r");
    char buf[1024];
    if (fgets (buf, sizeof(buf), stream)) {
      actual_signature = buf;
      actual_signature = trim(actual_signature);
    }
    pclose(stream);
    // See if the filename exists in the table anyway.
    bool update = false;
    int count = con.executeint32("select count(*) from '%s' where filename='%s';", TABLE_FILE_SIGNATURE, filename.c_str());
    if(count==0) {
    // Doesn't exists, so schedule the file for updating the database.
      update = true;
    }
    if (!update) {
      // The filename is in the database - now check its signature.
      sqlite3::reader reader = con.executereader("select signature from '%s' where filename='%s';", TABLE_FILE_SIGNATURE, filename.c_str());
      if(reader.read()) {
        string stored_signature = reader.getstring(0);
	    if (stored_signature != actual_signature) {
	      update = true;
	    }
	  }
	  reader.close();
    }
    if (update) {
      log ("Will be updated by " + filename);
	  // Store data about this filename.
	  Book wbook (filename, false);
	  for (unsigned int chapter = 1; chapter <= wbook.number_of_chapters; chapter++) {
	    vector<Glib::ustring> lines;
	    wbook.get_chapter (chapter, lines);
        update_chapter (wbook.get_id_index(), chapter, lines);
	  }
	  // Store its signature.
	  store_signature(filename, actual_signature);
    }
    con.close();
  }
  catch(exception &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
  catch(Glib::Error &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
}


void Worddatabase::delete_book(unsigned int book) {
  try {
    sqlite3::connection con(dbfilename().c_str());
    con.executenonquery("begin;");
    con.executenonquery("delete from '%s' where book = %d;", TABLE_LINE_AS_IS, book);
    con.executenonquery("delete from '%s' where book = %d;", TABLE_LINE_CASEFOLDED, book);
    con.executenonquery("delete from '%s' where book = %d;", TABLE_WORD_AS_IS, book);
    con.executenonquery("delete from '%s' where book = %d;", TABLE_WORD_CASEFOLDED, book);
    con.executenonquery("commit;");
    con.close();
  }
  catch(exception &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
  catch(Glib::Error &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
}


string Worddatabase::dbfilename() {
  // Returns full path and name of the word database.
  return Glib::build_filename(LOCALSTATEPATH, myproject + "_wordsdatabase.sqlite3");
}


void Worddatabase::store_signature(const string& filename, const string& signature) {
  // Store the signature of the book.
  try {
    sqlite3::connection con(dbfilename().c_str());
    con.executenonquery("begin;");
    con.executenonquery("delete from '%s' where filename = '%s';", TABLE_FILE_SIGNATURE, filename.c_str());
    con.executenonquery("insert into '%s' values('%s', '%s');", TABLE_FILE_SIGNATURE, filename.c_str(), signature.c_str());
    con.executenonquery("commit;");
    con.close();
  }
  catch(exception &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
  catch(Glib::Error &ex) {
    cerr << "Error: " << ex.what() << endl;
  }
}


